"""Print today's date and a calendar for current month."""

from calendar import TextCalendar
from datetime import date

t = date.today()
print("Today is", t)
print()  # prints empty line
TextCalendar().prmonth(t.year, t.month)

# Excercises:
# 1. change output for today's date
# 2. print calendar for the next month as well
# 3. change the first day the week in calendar
# 4. print your birthday
# 5. find out what day of the week your actual birthday was
# 6. implement print_month(year, month) function

# References:
# - https://docs.python.org/3/library/datetime.html#available-types
# - https://docs.python.org/3/library/calendar.html#calendar.TextCalendar



import ast
import subprocess
import sys
from dataclasses import dataclass
from pathlib import Path
from typing import ClassVar

from rich.console import Console
from rich.markdown import Markdown
from rich.syntax import Syntax

__all__ = ["CodeFile", "MarkdownFile", "print_md", "ls", "here"]


def print_md(text: str, use_pager=False):
    console = Console(width=80)
    md = Markdown(text)
    if use_pager:
        with console.pager():
            console.print(md)
    else:
        console.print(md)


def print_code(filename: str):
    console = Console(width=80)
    syntax = Syntax.from_path(filename)
    console.print(syntax)


def here() -> Path:
    return Path(__file__).parent


def get_docstring(file_contents: str):
    module = ast.parse(file_contents)
    docstring = ast.get_docstring(module)
    return docstring if docstring else ""


def ls():
    for path in CodeFile("any").path.parent.iterdir():
        if path.is_file():
            print(path.name + "\t" + get_docstring(path.read_text()))


@dataclass
class File:
    filename: str
    ext: ClassVar[str]
    folder: ClassVar[str]

    def __post_init__(self):
        if not self.filename.endswith(self.ext):
            self.filename += "." + self.ext

    def assert_exists(self):
        if not self.path.exists():
            sys.exit("File not found " + str(self.path))
        return self

    @property
    def path(self):
        return here() / self.folder / self.filename

    @property
    def contents(self):
        return self.path.read_text()


@dataclass
class CodeFile(File):
    filename: str
    ext: ClassVar[str] = "py"
    folder: ClassVar[str] = "code"

    def run(self):
        subprocess.run(["python", self.path.absolute()])

    def print(self):
        print_code(self.path)


@dataclass
class MarkdownFile(File):
    filename: str
    ext: ClassVar[str] = "md"
    folder: ClassVar[str] = "topics"

    def print(self, paginate: bool):
        print_md(self.contents, paginate)



"""Python course for beginners - the command line part."""

import sys
from random import choice
from typing import Optional

import typer
from rich import print
from typer import Typer
from typing_extensions import Annotated

from lamr.file_handlers import CodeFile, MarkdownFile, here, ls, print_md

lamr_app = Typer(
    add_completion=False,
    help="Python course for beginners, on the command line.",
)


def print_from_root(filename):
    return print_md((here().parent / filename).read_text())


@lamr_app.command()
def about(contributors: bool = False, dev: bool = False):
    """Print a README file."""
    if contributors:
        print("""Contributors: ...""")
    elif dev:
        print_from_root("development.md")
    else:
        print_from_root("README.md")


@lamr_app.command()
def code(
    filename: Annotated[Optional[str], typer.Argument()] = None,
    list_: Annotated[bool, typer.Option("--list")] = False,
):
    """Show code example."""
    if filename is not None:
        CodeFile(filename).assert_exists().print()
    if filename is None or list_:
        ls()


@lamr_app.command()
def run(filename: str):
    """Run code example."""
    CodeFile(filename).assert_exists().run()


# TRY: console.pager() loses color maybe use scrolling by section
#      add Press any key to continue...
def show(md_file: str, paginate: bool = False):
    """Show page."""
    MarkdownFile(md_file).print(paginate)


topic_list = [
    "what-is-a-program",
    "run-your-code",
    "print",
    "value",
    "numbers",
    "strings",
    "bools",
    "expression",
    "variables",
    "input",
    "if-else",
    "lists",
    "for-loop",
    "while-loop",
    "functions",
    "types",
]

tree = dict(variables=["variable-assignment.md"])


def print_list(header: str, items: list[str]):
    eol = "\n  "
    print(header + ":" + eol, eol.join(items), sep="")


@lamr_app.command()
def learn(
    topic: Annotated[Optional[str], typer.Argument()] = None, random: bool = False
):
    """Learn or review a topic."""
    if topic in tree.keys():
        for md_file in tree[topic]:
            show(md_file)
        sys.exit(0)
    if random:
        learn(choice(list(tree.keys())))  # mypy wants it this way
    print_list("Available topics", list(tree.keys()))  # mypy wants it this way
    print_list("Tentative topics", topic_list)


# @lamr_app.command()
# def book():
#     """Create a single file."""

# @lamr_app.command()
# def resources():
#     """List more learning resources."""
